from gpt_do.doers.doer import Doer
from gpt_do.vendor.chatgpt_wrapper.chatgpt_wrapper import ChatGPT


class PywrightDoer(Doer):
    def load_bot(self):
        bot = ChatGPT(headless=not self.debug)
        bot.conversation_id = self.state.get("conversation_id")
        bot.parent_message_id = self.state.get("parent_message_id")
        return bot

    def prime_convo(self):
        self.prompt += (
            "\nYou do not need to execute the commands, only print them."
            + " Does that make sense? Respond 'yes' if so."
        )
        self.dprint(self.prompt)

        if self.state:
            return

        response = self.ask(self.prompt).lower()
        self.dprint(response)

        assert "yes" in response

    def _ask(self, prompt: str) -> str:
        return self.bot.ask(prompt)

    def save_state(self):
        self.state["conversation_id"] = self.bot.conversation_id
        self.state["parent_message_id"] = self.bot.parent_message_id
        super().save_state()



import os
from textwrap import dedent

import click
import openai

from gpt_do.doers.doer import Doer


class GPT3Doer(Doer):
    model = "text-davinci-003"

    def load_bot(self):
        if not os.getenv("OPENAI_API_KEY"):
            raise click.UsageError(
                dedent(
                    """
                    To use {model}, you need to set an API key for OpenAI.
                    You can get one here: https://beta.openai.com/account/api-keys.
                    Make sure to expose it in your env as OPENAI_API_KEY.
                    """
                ).format(model=self.model)
            )

    def prime_convo(self):
        self.dprint(self.prompt)

    def _ask(self, prompt: str) -> str:
        contents = openai.Completion.create(
            engine=self.__class__.model,
            prompt=f"{self.prompt}\nRequest: {prompt}\n{{",
            stop="}\n",
            temperature=0,
            max_tokens=256,
        )
        self.dprint(contents)
        return "{" + contents.choices[0]["text"] + "}"

    def check_cache(self, key):
        return self.state.get("cache", {}).get(f"{self.model}:{key}")

    def update_cache(self, key, value):
        cache = self.state["cache"] = self.state.get("cache", {})
        cache[f"{self.model}:{key}"] = value



import json
import os
import sys
import tempfile
from abc import ABC, abstractmethod
from functools import cached_property
from pathlib import Path
from textwrap import dedent
from typing import Any, Literal, Union, overload

import click
import dirtyjson
from retry import retry


class Doer(ABC):
    PROMPT = dedent(
        """
        You are going to translate a natural language description to a series of commands for the {shell} shell.

        Here is some information about the environment: {uname}

        You should respond with a JSON object that has the keys "commands" and "explanation'.
        "commands" is an array of strings representing shell commands. If the request would require multiple commands, respond with all the required commands in the array.
        "explanation" is a string which at most one sentence, and cannot contain any other commands. The explanation should help the user understand what the commands are going to do.

        Here is an example:

        {{
            "commands" : ["ls", "cat", ...],
            "explanation" : "These commands will...",
        }}

        Do not respond with anything other than this JSON. Your response should be valid JSON. Do not include any notes.
        Do not rename the keys "commands" or "explanation".
    """
    )

    def __init__(self, debug: bool = False) -> None:
        self.shell_path = os.getenv("SHELL", "/usr/bin/bash")
        self.shell = os.path.split(self.shell_path)[-1]
        self.prompt = self.PROMPT.format(shell=self.shell, uname=repr(list(os.uname())))

        self.debug = debug
        self.state = self.load_state()

        self.bot = self.load_bot()

        self.prime_convo()

    def dprint(self, *args):
        if self.debug:
            print(*args, file=sys.stderr)

    @abstractmethod
    def load_bot(self) -> Any:
        ...

    @abstractmethod
    def _ask(self, prompt: str) -> str:
        ...

    @abstractmethod
    def prime_convo(self):
        ...

    @cached_property
    def state_path(self) -> Path:
        path = Path(os.getenv("XDG_CACHE_HOME", "~/.cache")).expanduser() / "do"
        path.mkdir(parents=True, exist_ok=True)
        return path / "state.json"

    def load_state(self):
        if not self.state_path.exists():
            return {}
        else:
            return json.load(self.state_path.open())

    def save_state(self):
        json.dump(self.state, self.state_path.open("w"))

    @overload
    def ask(self, prompt: str) -> str:
        ...

    @overload
    def ask(self, prompt: str, is_json: Literal[True]) -> dict:
        ...

    def ask(self, prompt: str, is_json: bool = False) -> Union[str, dict]:
        resp = self._ask(prompt)
        if not is_json:
            return resp

        try:
            resp = dirtyjson.loads(resp)
        except Exception:
            self.dprint(resp)
            raise click.UsageError("GPT returned an invalid response. Try again?")

        return resp

    def check_cache(self, key):
        return self.state.get("cache", {}).get(key)

    def update_cache(self, key, value):
        cache = self.state["cache"] = self.state.get("cache", {})
        cache[key] = value

    @retry(tries=3, delay=0.5)
    def query(self, query) -> dict:
        if cached := self.check_cache(query):
            return cached

        resp = self.ask(query, is_json=True)
        self.update_cache(query, resp)
        self.save_state()
        return resp

    def execute(self, commands: list) -> None:
        f = tempfile.NamedTemporaryFile(suffix=f".{self.shell}")
        f.write("\n".join(commands).encode("utf-8"))
        f.flush()
        os.execl(self.shell_path, "-c", f.name)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gpt_do', 'gpt_do.doers']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'dirtyjson>=1.0.8,<2.0.0',
 'openai>=0.25.0,<0.26.0',
 'playwright>=1.29.0,<2.0.0',
 'retry>=0.9.2,<0.10.0',
 'rich>=12.6.0,<13.0.0']

entry_points = \
{'console_scripts': ['ddo = gpt_do.cli:do',
                     'do = gpt_do.cli:do',
                     'gpt-do = gpt_do.cli:do']}

setup_kwargs = {
    'name': 'gpt-do',
    'version': '0.1.14',
    'description': 'GPT-powered bash commands.',
    'long_description': '# `gpt-do`\n\nThis is a handy-dandy CLI for when you don\'t know wtf to do.\n\nInstead of furiously grepping through man pages, simply use `do` (or `ddo` if on `bash`/`zsh`), and have GPT-3 do all the magic for you.\n\nCheck out the blog post [here](https://musings.yasyf.com/never-write-a-bash-command-again-with-gpt-3/).\n\n## Demo\n\nClick to play:\n\n[![asciicast](https://asciinema.org/a/oXRkVfVsxvUFq4SFjrstgsZck.png)](https://asciinema.org/a/oXRkVfVsxvUFq4SFjrstgsZck?i=0.5&autoplay=1)\n\n## Installation\n\nWe recommend using [`pipx`](https://pypa.github.io/pipx/):\n\n```console\n$ pipx install gpt-do\n$ which do\n~/.local/bin/do\n```\n\nHowever you can also use `brew`:\n\n```console\n$ brew install yasyf/do/do\n$ which do\n/opt/homebrew/bin/do\n```\n\nOr `pip`:\n\n```console\n$ pip install gpt-do\n$ which do\n~/.asdf/installs/python/3.11.0/bin/do\n```\n\n## Usage\n\n**n.b.** If you\'re on `bash` or `zsh`, `do` is a reserved keyword, so you\'ll have to use the alias `ddo`.\n\n\n```console\n$ export OPENAI_API_KEY=xxx # stick this in your bash_profile\n$ do amend the message of my last commit to "It works!"\nThis command will amend the message of the last commit to \'It works!\'.\ngit commit --amend -m \'It works!\'\nDo you want to continue? [y/N]: y\n[main 3e6a2f6] It works!!\n Date: Thu Dec 22 01:15:40 2022 -0800\n 5 files changed, 1088 insertions(+)\n create mode 100644 .gitignore\n create mode 100644 .gitmodules\n create mode 100644 README.md\n create mode 100644 poetry.lock\n create mode 100644 pyproject.toml\n```\n',
    'author': 'Yasyf Mohamedali',
    'author_email': 'yasyfm@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/yasyf/gpt-do',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9.0,<4.0.0',
}


setup(**setup_kwargs)
